@@ -18,7 +18,7 @@
  *
  */
 
-#include <sound/driver.h>
+#include "adriver.h"
 #include <linux/time.h>
 #include <linux/slab.h>
 #include <linux/compat.h>
@@ -406,7 +406,7 @@ static int snd_pcm_info_ioctl32(unsigned int fd, unsigned int cmd, unsigned long
 	substream = pcm_file->substream;
 	if (! substream)
 		return -ENOTTY;
-	substream->no_mmap_ctrl = 1;
+	pcm_file->no_compat_mmap = 1;
 	return filp->f_op->ioctl(filp->f_dentry->d_inode, filp, cmd, arg);
 }
 
@@ -436,6 +436,7 @@ struct ioctl32_mapper pcm_mappers[] = {
 	/* MAP_COMPAT(SNDRV_PCM_IOCTL_INFO), */
 	{ SNDRV_PCM_IOCTL_INFO, snd_pcm_info_ioctl32 },
 	MAP_COMPAT(SNDRV_PCM_IOCTL_TSTAMP),
+	MAP_COMPAT(SNDRV_PCM_IOCTL_TTSTAMP),
 	{ SNDRV_PCM_IOCTL_HW_REFINE32, AP(pcm_hw_refine) },
 	{ SNDRV_PCM_IOCTL_HW_PARAMS32, AP(pcm_hw_params) },
 	MAP_COMPAT(SNDRV_PCM_IOCTL_HW_FREE),
