@@ -1,6 +1,6 @@
+#include "alsa-autoconf.h"
 #define __NO_VERSION__
-#include <linux/config.h>
-#include <linux/version.h>
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,12)
 #include <linux/compiler.h>
 #ifndef __iomem
@@ -21,10 +21,20 @@
 #ifndef __safe
 #define __safe
 #endif
+#ifndef __bitwise
+#define __bitwise
+#endif
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
-#include <sound/driver.h>
+#include "adriver.h"
 #endif /* KERNEL < 2.6.0 */
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,21)
+#include <linux/compiler.h>
+#ifndef __printf
+#define __printf(a,b)   /*nop*/
+#endif
+#endif
+
 #include "../alsa-kernel/core/memory.c"
