@@ -1,9 +1,7 @@
+#include "alsa-autoconf.h"
 #define __NO_VERSION__
-#include "config.h"
-
-#include <linux/version.h>
-#include <linux/config.h>
 #include <linux/string.h>
+#include <linux/sched.h>
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 #if defined(CONFIG_MODVERSIONS) && !defined(__GENKSYMS__) && !defined(__DEPEND__)
@@ -11,6 +9,7 @@
 #endif
 #endif
 
+#include <linux/module.h>
 #include <linux/kmod.h>
 #include <linux/devfs_fs_kernel.h>
 
@@ -42,6 +41,7 @@ size_t snd_compat_strlcat(char *dest, const char *src, size_t count)
 	dest[len] = 0;
 	return res;
 }
+EXPORT_SYMBOL(snd_compat_strlcat);
 #endif
 
 #ifndef CONFIG_HAVE_VSNPRINTF
@@ -89,6 +89,7 @@ int snd_compat_vsscanf(const char *buf, const char *fmt, va_list args)
 	}
 	return num;
 }
+EXPORT_SYMBOL(snd_compat_vsscanf);
 
 int snd_compat_sscanf(const char *buf, const char *fmt, ...)
 {
@@ -100,6 +101,7 @@ int snd_compat_sscanf(const char *buf, const char *fmt, ...)
 	va_end(args);
 	return res;
 }
+EXPORT_SYMBOL(snd_compat_sscanf);
 #endif
 
 #ifdef CONFIG_HAVE_OLD_REQUEST_MODULE
@@ -115,6 +117,7 @@ void snd_compat_request_module(const char *fmt, ...)
 		request_module(buf);
 	va_end(args);
 }
+EXPORT_SYMBOL(snd_compat_request_module);
 #endif
 
 #if defined(CONFIG_DEVFS_FS)
@@ -136,6 +139,7 @@ void snd_compat_devfs_remove(const char *fmt, ...)
 	}
 	va_end(args);
 }
+EXPORT_SYMBOL(snd_compat_devfs_remove);
 
 #endif /* 2.5.29 */
 
@@ -159,6 +163,7 @@ int snd_compat_devfs_mk_dir(const char *dir, ...)
 	}
 	return 0;
 }
+EXPORT_SYMBOL(snd_compat_devfs_mk_dir);
 
 extern struct file_operations snd_fops;
 int snd_compat_devfs_mk_cdev(dev_t dev, umode_t mode, const char *fmt, ...)
@@ -177,6 +182,7 @@ int snd_compat_devfs_mk_cdev(dev_t dev, umode_t mode, const char *fmt, ...)
 	}
 	return 0;
 }
+EXPORT_SYMBOL(snd_compat_devfs_mk_cdev);
 
 #endif /* 2.5.67 */
 
@@ -197,6 +203,7 @@ int snd_pci_dev_present(const struct pci_device_id *ids)
 	}
 	return 0;
 }
+EXPORT_SYMBOL(snd_pci_dev_present);
 #endif
 
 /*
@@ -213,6 +220,7 @@ void snd_compat_msleep(unsigned int msecs)
 		timeout = schedule_timeout(timeout);
 	}
 }
+EXPORT_SYMBOL(snd_compat_msleep);
 #endif
 
 #ifndef CONFIG_HAVE_MSLEEP_INTERRUPTIBLE
@@ -227,4 +235,121 @@ unsigned long snd_compat_msleep_interruptible(unsigned int msecs)
 	}
 	return (timeout * 1000) / HZ;
 }
+EXPORT_SYMBOL(snd_compat_msleep_interruptible);
 #endif /* < 2.6.6 */
+
+/* wrapper for new irq handler type */
+#ifndef CONFIG_SND_NEW_IRQ_HANDLER
+#include <linux/interrupt.h>
+#include <linux/mutex.h>
+#include <linux/list.h>
+#include <linux/slab.h>
+typedef int (*snd_irq_handler_t)(int, void *);
+struct irq_list {
+	snd_irq_handler_t handler;
+	int irq;
+	void *data;
+	struct list_head list;
+};
+	
+struct pt_regs *snd_irq_regs;
+EXPORT_SYMBOL(snd_irq_regs);
+
+#if defined(IRQ_NONE) && LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
+static irqreturn_t irq_redirect(int irq, void *data, struct pt_regs *reg)
+{
+	struct irq_list *list = data;
+	irqreturn_t val;
+	snd_irq_regs = reg;
+	val = list->handler(irq, list->data);
+	snd_irq_regs = NULL;
+	return val;
+}
+#else
+static void irq_redirect(int irq, void *data, struct pt_regs *reg)
+{
+	struct irq_list *list = data;
+	snd_irq_regs = reg;
+	list->handler(irq, list->data);
+	snd_irq_regs = NULL;
+}
+#endif
+
+static LIST_HEAD(irq_list_head);
+static DEFINE_MUTEX(irq_list_mutex);
+
+int snd_request_irq(unsigned int irq, snd_irq_handler_t handler,
+		    unsigned long irq_flags, const char *str, void *data)
+{
+	struct irq_list *list = kmalloc(sizeof(*list), GFP_KERNEL);
+	int err;
+
+	if (!list)
+		return -ENOMEM;
+	list->handler = handler;
+	list->irq = irq;
+	list->data = data;
+	err = request_irq(irq, irq_redirect, irq_flags, str, list);
+	if (err) {
+		kfree(list);
+		return err;
+	}
+	mutex_lock(&irq_list_mutex);
+	list_add(&list->list, &irq_list_head);
+	mutex_unlock(&irq_list_mutex);
+	return 0;
+}
+EXPORT_SYMBOL(snd_request_irq);
+
+void snd_free_irq(unsigned int irq, void *data)
+{
+	struct list_head *p;
+
+	mutex_lock(&irq_list_mutex);
+	list_for_each(p, &irq_list_head) {
+		struct irq_list *list = list_entry(p, struct irq_list, list);
+		if (list->irq == irq && list->data == data) {
+			free_irq(irq, list);
+			list_del(p);
+			kfree(list);
+			break;
+		}
+	}
+	mutex_unlock(&irq_list_mutex);
+}
+EXPORT_SYMBOL(snd_free_irq);
+#endif /* !CONFIG_SND_NEW_IRQ_HANDLER */
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 33)
+#include <linux/ctype.h>
+
+char *compat_skip_spaces(const char *str)
+{
+	while (isspace(*str))
+		++str;
+	return (char *)str;
+}
+EXPORT_SYMBOL(compat_skip_spaces);
+#endif /* < 2.6.33 */
+
+#ifndef CONFIG_GCD
+#include <linux/gcd.h>
+#ifdef CONFIG_SND_COMPAT_GCD
+/* Greatest common divisor */
+unsigned long gcd(unsigned long a, unsigned long b)
+{
+	unsigned long r;
+	if (a < b) {
+		r = a;
+		a = b;
+		b = r;
+	}
+	while ((r = a % b) != 0) {
+		a = b;
+		b = r;
+	}
+	return b;
+}
+EXPORT_SYMBOL(gcd);
+#endif /* CONFIG_SND_COMPAT_GCD */
+#endif /* !CONFIG_GCD */
