@@ -15,7 +15,7 @@
  * 2005-10	Some finishing -- Giorgio Padrin
  */
 
-#include <sound/driver.h>
+#include "adriver.h"
 
 #include <linux/module.h>
 #include <linux/init.h>
@@ -291,7 +291,11 @@ static int snd_pxa2xx_i2sound_ngn_set_rate(unsigned int rate)
 	return 0;
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 19)
+static void snd_pxa2xx_i2sound_ngn_dma_irq(int ch, void *data)
+#else
 static void snd_pxa2xx_i2sound_ngn_dma_irq(int ch, void *data, struct pt_regs *regs)
+#endif
 {
 	struct snd_pxa2xx_i2sound_stream *s = (struct snd_pxa2xx_i2sound_stream*) data;
 	u32 dcsr;
@@ -856,11 +860,10 @@ int snd_pxa2xx_i2sound_card_activate(struct snd_pxa2xx_i2sound_board *board)
 	if ((ret = snd_pxa2xx_i2sound_board_activate()) < 0) goto failed_activate_board;
 
 	/* new ALSA card */
-	if ((card.acard = snd_card_new(-1, card.board->acard_id, THIS_MODULE, 0)) == NULL) {
-		card.acard = NULL;
-		ret = -ENOMEM;
+	ret = snd_card_create(-1, card.board->acard_id, THIS_MODULE, 0,
+			      &card.acard);
+	if (ret < 0)
 		goto failed_new_acard;
-	}
 
 	strlcpy(card.acard->driver, card.board->acard_id, sizeof(card.acard->driver));
 	strlcpy(card.acard->shortname, card.name, sizeof(card.acard->shortname));
