@@ -304,6 +304,19 @@
 #ifndef PCI_DEVICE_ID_CREATIVE_EMU10K1
 #define PCI_DEVICE_ID_CREATIVE_EMU10K1	0x0002
 #endif
+#ifndef PCI_DEVICE_ID_CREATIVE_20K1
+#define PCI_DEVICE_ID_CREATIVE_20K1	0x0005
+#define PCI_DEVICE_ID_CREATIVE_20K2	0x000b
+#define PCI_SUBDEVICE_ID_CREATIVE_SB0760	0x0024
+#define PCI_SUBDEVICE_ID_CREATIVE_SB08801	0x0041
+#define PCI_SUBDEVICE_ID_CREATIVE_SB08802	0x0042
+#define PCI_SUBDEVICE_ID_CREATIVE_SB08803	0x0043
+#define PCI_SUBDEVICE_ID_CREATIVE_HENDRIX	0x6000
+#endif
+
+#ifndef PCI_SUBDEVICE_ID_CREATIVE_SB1270
+#define PCI_SUBDEVICE_ID_CREATIVE_SB1270 0x0062
+#endif
 
 /* ice1712 */
 #ifndef PCI_VENDOR_ID_ICE
@@ -399,3 +412,22 @@
 #ifndef PCI_VENDOR_ID_DFI
 #define PCI_VENDOR_ID_DFI		0x15bd
 #endif
+
+#ifndef PCI_DEVICE_ID_ATI_SBX00_SMBUS
+#define PCI_DEVICE_ID_ATI_SBX00_SMBUS	0x4385
+#endif
+
+/* lola */
+#ifndef PCI_VENDOR_ID_DIGIGRAM
+#define PCI_VENDOR_ID_DIGIGRAM		0x1369
+#endif
+
+/* ad1889 */
+#ifndef PCI_VENDOR_ID_ANALOG_DEVICES
+#define PCI_VENDOR_ID_ANALOG_DEVICES	0x11d4
+#endif
+
+#ifndef PCI_DEVICE_ID_AD1889JS
+#define PCI_DEVICE_ID_AD1889JS		0x1889
+#endif
+
