@@ -1,7 +1,6 @@
 #define __NO_VERSION__
 
-#include <linux/config.h>
-#include <linux/version.h>
+#include "adriver.h"
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0)
 #ifdef __i386__
