@@ -1,6 +1,5 @@
 #define __NO_VERSION__
-#include <linux/config.h>
-#include <linux/version.h>
+#include "adriver.h"
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0)
 #define CONFIG_ADB_CUDA
 #define CONFIG_ADB_PMU
