@@ -1,3 +1,3 @@
-#include "adriver.h"
+#include "../usbaudio.inc"
 #include "../../alsa-kernel/usb/misc/ua101.c"
 EXPORT_NO_SYMBOLS;
