@@ -1,6 +1,6 @@
 #!/bin/bash
 
-version=0.2.1
+version=0.2.4
 protocol=
 distrib=unknown
 distribver=0.0
@@ -153,6 +153,9 @@ do
 			url="$2"
 			shift ;;
 		esac
+		if test -d "$url"; then
+			url="file:///$url"
+		fi
 		;;
 	--git*)
 		case "$#,$1" in
@@ -182,7 +185,7 @@ do
 	--install)
 		install=true
 		;;
-	--patch)
+	--patch*)
 		case "$#,$1" in
 		*,*=*)
 			patch=`expr "z$1" : 'z-[^=]*=\(.*\)'` ;;
@@ -199,12 +202,10 @@ do
 		pkg=${pkg:2}
 		package="alsa-$pkg"
 		packagedefault=
-		test -z "$url" && url="$baseurl$package"
 		;;
 	--run)
 		package="alsa-lib"
 		packagedefault=
-		test -z "$url" && url="$baseurl$package"
 		shift
 		runargs="$@"
 		break
@@ -285,8 +286,26 @@ question_bool() {
 
 # Set $distrib and $distribver
 check_distribution() {
-	distrib=$(lsb_release -ds 2> /dev/null | cut -d ' ' -f 1)
-	distrib=${distrib:1}
+	distrib=$(lsb_release -ds 2> /dev/null)
+	if test -z "$distrib"; then
+		if test -f /etc/redhat-release; then
+			distrib="Fedora"
+			install_package lsb
+			distrib=
+		fi
+		distrib=$(lsb_release -ds 2> /dev/null)
+		distrib=$(echo $distrib)
+	fi
+	local tmp=${distrib:0:25}
+	if test "$tmp" == "\"Red Hat Enterprise Linux"; then
+		distrib="RHEL"
+	else
+		distrib=$(echo $distrib | cut -d ' ' -f 1)
+	fi
+	tmp=${distrib:0:1}
+	if test "$tmp" = "\""; then
+		distrib=${distrib:1}
+	fi
 	distribver=$(lsb_release -rs 2> /dev/null)
 	if test -z "$distrib"; then
 		echo >&2 "Unable to determine Linux distribution."
@@ -310,26 +329,74 @@ is_rpm_installed() {
 	fi
 }
 
+# Echo "true" if deb installed, else "false"
+is_deb_installed() {
+    dpkg -l $1 2>&1 | grep -Eq '^ii'
+       if test "$?" = "0"; then
+               echo "true"
+       else
+               echo "false"
+       fi
+}
+
 # Install package
 # $1 is package name
 install_package() {
-	echo "Trying to install package '$1':"
+	pkg="$1"
 	case "$distrib" in
 	openSUSE)
-		if zypper install $1; then
-			echo >&2 "Cannot install package '$1'."
-			exit 1
-		fi
+		test "$pkg" == "alsa-lib-devel" && pkg="alsa-devel"
 		;;
-	Fedora)
-		yum install $1
+	Fedora|RHEL)
+		test "$pkg" == "lsb" && pkg="redhat-lsb"
+		;;
+	Debian|Ubuntu)
+		test "$pkg" == "lsb" && pkg="lsb-release"
+		test "$pkg" == "git" && pkg="git-core"
+		test "$pkg" == "alsa-lib-devel" && pkg="libasound2-dev"
+		test "$pkg" == "gettext-tools" && pkg="gettext-base"
+		test "$pkg" == "gettext-devel" && pkg="gettext"
+		test "$pkg" == "ncurses-devel" && pkg="ncurses-bin"
 		;;
 	*)
 		echo >&2 "Cannot install $1 for unsupported distribution $distrib."
 		exit 1
 	esac
-	if test $(is_rpm_installed $1) = "false" ; then
-		echo >&2 "Package '$1' was not installed."
+	echo "Trying to install package '$pkg':"
+	case "$distrib" in
+	openSUSE)
+		zypper install $pkg
+		;;
+	Fedora|RHEL)
+		yum install -y $pkg
+		;;
+	Debian|Ubuntu)
+		apt-get install -y $pkg
+		;;
+	*)
+		echo >&2 "Cannot install $pkg for unsupported distribution $distrib."
+		exit 1
+	esac
+	if test $(is_rpm_installed $pkg) = "false" -a "$pkg" = "git"; then
+		case "$distrib" in
+		RHEL)
+			install_package perl-DBI
+			rm -f $tmpdir/git.rpm
+			ver="1.7.2.2-1.el${distribver:0:1}.rf"
+			wget -O $tmpdir/git.rpm http://packages.sw.be/git/perl-Git-$ver.$(uname -m).rpm
+			if test -s $tmpdir/git.rpm; then
+				rpm -Uvh --nodeps $tmpdir/git.rpm
+			fi
+			wget -O $tmpdir/git.rpm http://packages.sw.be/git/git-$ver.$(uname -m).rpm
+			if test -s $tmpdir/git.rpm; then
+				rpm -Uvh $tmpdir/git.rpm
+			fi
+			rm -f $tmpdir/git.rpm
+			;;
+		esac
+	fi
+	if test $(is_rpm_installed $pkg) = "false" ; then
+		echo >&2 "Package '$pkg' was not installed."
 		exit 1
 	fi
 	echo "  installed"
@@ -344,7 +411,7 @@ check_kernel_source() {
 			install_package kernel-source
 		fi
 		;;
-	Fedora)
+	Fedora|RHEL)
 		if uname --kernel-release | grep -q '\.PAE$'; then
 			local kernel_devel=kernel-PAE-devel
 		else
@@ -354,6 +421,12 @@ check_kernel_source() {
 			install_package $kernel_devel
 		fi
 		;;
+	Debian|Ubuntu)
+		local kernel_devel="linux-source-$(uname --kernel-release | sed 's/-.*$//')"
+		if test $(is_deb_installed $kernel_devel) = "false" ; then
+			install_package $kernel_devel
+		fi
+		;;
 	*)
 		echo >&2 "Cannot install kernel sources for unsupported distribution $distrib."
 		exit 1
@@ -464,13 +537,13 @@ check_compilation_environment() {
 		else
 			echo "Program diff found."
 		fi
-		if test "$protocol" = "git"; then
-			local a=$(git --version | head -1 | cut -d ' ' -f 1)
-			if test "$a" != "git"; then
-				install_package git
-			else
-				echo "Program git found."
-			fi
+		local a=$(make --version | head -1 | cut -d ' ' -f 1)
+		local b=$(make --version | head -1 | cut -d ' ' -f 2)
+		if test "$a" != "make" -a "$b" != "make" -a \
+		   "$a" != "Make" -a "$b" != "Make"; then
+			install_package make
+		else
+			echo "Program make found."
 		fi
 		check_kernel_source
 		echo "httpdownloader=$httpdownloader" >> $env
@@ -479,6 +552,70 @@ check_compilation_environment() {
 		echo "File $env is cached."
 		. $env
 	fi
+	if test -z "$present_tool_git"; then
+		if test "$protocol" = "git"; then
+			local a=$(git --version | head -1 | cut -d ' ' -f 1)
+			if test "$a" != "git"; then
+				install_package git
+			else
+				echo "Program git found."
+				echo "present_tool_git=y" >> $env
+			fi
+		fi
+	fi
+	if test -z "$present_tool_gettext"; then
+		if test "$package" = "alsa-utils"; then
+			local a=$(gettext --version | head -1 | cut -d ' ' -f 1)
+			if test "$a" != "gettext"; then
+				install_package gettext-tools
+				local a=$(gettextize --version | head -1 | cut -d ' ' -f 1)
+				if test -z "$a"; then
+					install_package gettext-devel
+				fi 
+			else
+				local a=$(gettextize --version | head -1 | cut -d ' ' -f 1)
+				if test -z "$a"; then
+					install_package gettext-devel
+				else
+					echo "Program gettext found."
+					echo "present_tool_gettext=y" >> $env
+				fi
+			fi
+		fi
+	fi
+	if test -z "$present_tool_xmlto"; then
+		if test "$package" = "alsa-utils"; then
+			local a=$(xmlto --version | head -1 | cut -d ' ' -f 1)
+			if test "$a" != "xmlto"; then
+				install_package xmlto
+			else
+				echo "Program xmlto found."
+				echo "present_tool_xmlto=y" >> $env
+			fi
+		fi
+	fi
+	if test -z "$present_tool_alsa_lib"; then
+		if test "$package" = "alsa-utils"; then
+			if ! test -r "/usr/include/alsa/asoundlib.h"; then
+				install_package alsa-lib-devel
+			else
+				echo "Package alsa-lib-devel found."
+				echo "present_tool_alsa_lib=y" >> $env
+			fi
+		fi
+	fi
+	if test -z "$present_tool_ncurses"; then
+		if test "$package" = "alsa-utils"; then
+			local a=$(ncurses6-config --version | head -1 | cut -d '.' -f 1)
+			local b=$(ncurses5-config --version | head -1 | cut -d '.' -f 1)
+			if test "$a" != "5" -a "$a" != "6" -a "$b" != "5" -a "$b" != "6"; then
+				install_package ncurses-devel
+			else
+				echo "Package ncurses-devel found."
+				echo "present_tool_ncurses=y" >> $env
+			fi
+		fi
+	fi
 }
 
 # Log and execute $@ and check success
@@ -543,15 +680,15 @@ do_compile() {
 	else
 		for patch in $patches; do
 			local pstrip=1
-			if ! test patch -s -p$pstrip -N --dry-run < $patch; then
+			if ! patch -s -p$pstrip -N --dry-run < $patch; then
 				local pstrip=0
-				if ! test patch -s -p$pstrip -N --dry-run < $patch; then
+				if ! patch -s -p$pstrip -N --dry-run < $patch; then
 					echo >&2 "Cannot apply patch $patch."
 					exit 1
 				fi
 			fi
 			echo "Applying patch $patch: "
-			if ! test patch -p$pstrip -N < $patch; then
+			if ! patch -p$pstrip -N < $patch; then
 				exit 1
 			fi
 		done
@@ -1055,6 +1192,7 @@ if test -n "$clean"; then
 		echo " success"
 	else
 		package_switch $clean
+		package="$clean"
 		echo -n "Removing package $package:"
 		rm "$tmpdir/environment.*" 2> /dev/null
 		if test "$package" = "alsa-driver"; then
@@ -1106,7 +1244,7 @@ if test -n "$kernelmodules" -a -z "$compile" -a -n "$tree" -a -n "$status"; then
 fi
 
 case "$protocol" in
-http|https|file)
+http|https|ftp|file)
 	check_compilation_environment
 	if test -n "$tree"; then
 		echo "$package tree $tree is present."
@@ -1158,6 +1296,7 @@ http|https|file)
 		echo "ALSA package $package was successfully built."
 		echo "Binaries are in $tmpdir/$tree directory."
 	fi
+	cd ..
 	;;
 git)
 	check_compilation_environment
@@ -1170,9 +1309,9 @@ git)
 			git_clone $url "alsa-kmirror"
 		fi
 		git_clone $url $package
-		echo "alsa-driver" > $packagedir
+		echo "$package" > $packagedir
 	fi
-	do_cmd cd alsa-driver
+	do_cmd cd $package
 	do_compile
 	status="compile"
 	echo $status > $packagestatus
@@ -1182,6 +1321,7 @@ git)
 		echo "ALSA package $package was successfully built."
 		echo "Binaries are in $tmpdir/$package directory."
 	fi
+	cd ..
 	;;
 *)
 	echo >&2 "Unknown protocol '$protocol'."
